/*
* Copyright (C) 2009, Oy L M Ericsson Ab, NomadicLab <pubsub@nomadiclab.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* Alternatively, this software may be distributed under the terms of the BSD
* license.
*
* See LICENSE and COPYING for more details.
*/

//#include <pcap.h>
//#include <net/ethernet.h>
//#include <libnet11/libnet.h>

#include <sys/types.h>

/* psirpd_net.h */
#include <sys/socket.h>
#include <net/if_dl.h>
#include <net/if.h>

#include <string.h>
#include <stdlib.h>
#include <netinet/in.h>

#ifdef PSIRP_PLA
#include "libpla.h"
#endif

/* libpsirp */
#include <sys/param.h>
#include <libpsirp.h>

#include "psirp_common_types.h"
#include "psirpd_types.h"
#include "psirp.h"
//#include "psirp_io.h"
//#include "psirp_debug.h"
//#include "psirpd_pl.h"
#include "psirpd_hdrs.h"
#include "psirpd_net.h"
#include "psirpd_out_q.h"
#include "psirpd_packet.h"
#include "psirpd_fwd_bf.h"
#include "psirpd_rzv.h"
#include "psirpd_sec_pla.h"
#include "../../libpsirp/src/psirp_debug.h"

static void psirpd_hdrs_init_hdr(struct pkt_ctx*, 
                                 psirpd_hdrs_t*,
                                 u_int32_t);

psirp_error_t
psirpd_hdrs_init(void) {
    return PSIRP_OK;
}

void
psirpd_hdrs_cleanup(void) {
    return;
}

static void
psirpd_hdrs_init_hdr(pkt_ctx_t *pkt_ctx,
                     psirpd_hdrs_t *hdr,
                     u_int32_t hdr_type) {
    ENTER();

    hdr->hdr_type = hdr_type;
    hdr->nxt_hdr = 0;

    if (pkt_ctx->prev_hdr) {
        pkt_ctx->prev_hdr->nxt_hdr = hdr_type;
    }

    pkt_ctx->prev_hdr = hdr;

    RETURN;
}

psirp_error_t
psirpd_init_fwd_hdr(pkt_ctx_t   *pkt_ctx,
                    psirp_fid_t *fidp) {
    psirpd_hdrs_fwhdr_t *hdr;
    u_int32_t  hdr_type = PSIRP_HDR_FWD_BF;
    u_int8_t   proto_ver = PSIRP_FWD_PROTO_BF;
    
    ENTER();

    pkt_ctx->fwd_hdr = (struct psirpd_hdrs_fwhdr*)pkt_ctx->offset;
    pkt_ctx->offset += sizeof(struct psirpd_hdrs_fwhdr);
    PSIRP_ETL((pkt_ctx->offset - pkt_ctx->pkt) > pkt_ctx->pkt_len, 
               PSIRP_FAIL_OUT_OF_BUFFER, PSIRP_DBG_ERR,
               "Offset '%d' higher than lenght '%d' of the buffer\n",
               pkt_ctx->offset - pkt_ctx->pkt, pkt_ctx->pkt_len);

    pkt_ctx->payload = pkt_ctx->offset;
    hdr =  pkt_ctx->fwd_hdr;
    
    psirpd_hdrs_init_hdr(pkt_ctx, (psirpd_hdrs_t*)hdr, hdr_type);

    if (fidp) {
#if 0
        hdr->fid = *fidp;
#else
	memcpy(&hdr->fid, fidp, sizeof(hdr->fid));
#endif
    } 

    hdr->ttl = 0xff;
    hdr->d = 0;
    hdr->proto_ver = proto_ver;

    RETURN PSIRP_OK;
}

psirp_error_t
psirpd_init_rzv_hdr(pkt_ctx_t  *pkt_ctx,
                    u_int32_t   hdr_type,
                    psirp_id_t *sidp,
                    psirp_id_t *ridp,    
                    u_int64_t   seqnum) {
    psirpd_hdrs_rzvhdr_t *hdr;
    u_int8_t  proto_ver = PSIRP_RZV_PROTO;

    ENTER();

    pkt_ctx->rzv_hdr = (struct psirpd_hdrs_rzvhdr*)pkt_ctx->offset;
    pkt_ctx->offset += sizeof(struct psirpd_hdrs_rzvhdr);
    PSIRP_ETL((pkt_ctx->offset - pkt_ctx->pkt) > pkt_ctx->pkt_len, 
               PSIRP_FAIL_OUT_OF_BUFFER, PSIRP_DBG_ERR,
               "Offset '%d' higher than lenght '%d' of the buffer\n",
               pkt_ctx->offset - pkt_ctx->pkt, pkt_ctx->pkt_len);

    pkt_ctx->payload = pkt_ctx->offset;
    hdr = pkt_ctx->rzv_hdr;

    psirpd_hdrs_init_hdr(pkt_ctx, (psirpd_hdrs_t*)hdr, hdr_type);

    if (sidp) {
#if 0
        hdr->sid = *sidp;
#else
	memcpy(&hdr->sid, sidp, sizeof(hdr->sid));
#endif
    }
    if (ridp) {
#if 0
        hdr->rid = *ridp;
#else
	memcpy(&hdr->rid, ridp, sizeof(hdr->rid));
#endif
    }

    hdr->seqnum = seqnum;
    hdr->proto_ver = proto_ver;

    RETURN PSIRP_OK;
}

psirp_error_t
psirpd_init_md_hdr(pkt_ctx_t   *pkt_ctx,
                   psirp_id_t  *sidp,
                   psirp_id_t  *ridp,
                   psirp_fid_t *fidp,
                   u_int64_t    sub_id, 
		   u_int64_t    len,          
                   u_int64_t    max_seqnum) {

    psirpd_hdrs_metadata_t *hdr;
    u_int32_t  hdr_type = PSIRP_HDR_MD;

    ENTER();

    pkt_ctx->md_hdr = (psirpd_hdrs_metadata_t*)pkt_ctx->offset;
    pkt_ctx->offset += sizeof(psirpd_hdrs_metadata_t);
    PSIRP_ETL((pkt_ctx->offset - pkt_ctx->pkt) > pkt_ctx->pkt_len, 
               PSIRP_FAIL_OUT_OF_BUFFER, PSIRP_DBG_ERR,
               "Offset '%d' higher than lenght '%d' of the buffer\n",
               pkt_ctx->offset - pkt_ctx->pkt, pkt_ctx->pkt_len);

    pkt_ctx->payload = pkt_ctx->offset;
    hdr = pkt_ctx->md_hdr;

    psirpd_hdrs_init_hdr(pkt_ctx, (psirpd_hdrs_t*)hdr, hdr_type);

    if (sidp) {
#if 0
        hdr->sid = *sidp;
#else
	memcpy(&hdr->sid, sidp, sizeof(hdr->sid));
#endif
    }

    if (ridp) {
#if 0
        hdr->rid = *ridp;
#else
	memcpy(&hdr->rid, ridp, sizeof(hdr->rid));
#endif
    }

    if (fidp) {
#if 0
        hdr->fid = *fidp;
#else
	memcpy(&hdr->fid, fidp, sizeof(hdr->fid));
#endif
    }

    hdr->sub_id = sub_id;
    hdr->len = len;
    PSIRP_ETL(hdr->len > PSIRP_MAX_PUB_SIZE, 
              PSIRP_FAIL_BUF_LEN,
              PSIRP_DBG_ERR, "hdr->len == %d", hdr->len);

    hdr->max_seqnum = max_seqnum;
#if 0
    PSIRP_ETL(hdr->max_seqnum > PSIRP_DATAMAP_SIZE, 
              PSIRP_FAIL_BUF_LEN, 
              PSIRP_DBG_ERR, "hdr->max_seqnum == %d", hdr->max_seqnum);
#endif

    hdr->rzvhdr_type = pkt_ctx->rzv_hdr->hdr_type;

    RETURN PSIRP_OK;
}

psirp_error_t
psirpd_init_pla_hdr(pkt_ctx_t *pkt_ctx) {

    psirpd_hdrs_pla_t *hdr;
    u_int32_t  hdr_type = PSIRP_HDR_SEC_PLA;

    ENTER();

    pkt_ctx->pla_hdr = (psirpd_hdrs_pla_t*)pkt_ctx->offset;
    pkt_ctx->offset += sizeof(psirpd_hdrs_pla_t);
    PSIRP_ETL((pkt_ctx->offset - pkt_ctx->pkt) > pkt_ctx->pkt_len, 
               PSIRP_FAIL_OUT_OF_BUFFER, PSIRP_DBG_ERR,
               "Offset '%d' higher than lenght '%d' of the buffer\n",
               pkt_ctx->offset - pkt_ctx->pkt, pkt_ctx->pkt_len);

    pkt_ctx->payload = pkt_ctx->offset;
    hdr =  pkt_ctx->pla_hdr;

    psirpd_hdrs_init_hdr(pkt_ctx, (psirpd_hdrs_t*)hdr, hdr_type);
    
    RETURN PSIRP_OK;
}

psirp_error_t
psirpd_init_payload(pkt_ctx_t *pkt_ctx, u_int8_t *addr, int len) {

    ENTER();

    /* TODO: memcpy should NOT be used here in the future.
     * Can be avoided when using writev. 
     * Currently we need a continuous memory area.*/
    memcpy(pkt_ctx->payload, addr, len);
    pkt_ctx->offset += len;
    
    if (pkt_ctx->prev_hdr) {
        pkt_ctx->prev_hdr->nxt_hdr = PSIRP_PAYLOAD;
    }

    RETURN PSIRP_OK;
}

psirp_error_t
psirpd_init_bitmap_hdr(pkt_ctx_t *pkt_ctx, psirpd_bmap_t *data_map)
{
    
    int size, left;
    u_int32_t  hdr_type = PSIRP_HDR_BITMAP;
    psirp_error_t err;
    uint8_t *tmpmap;
    int maxsize;

    ENTER();

    pkt_ctx->bmap_hdr = (struct psirpd_hdrs_bitmap*)pkt_ctx->offset;
    pkt_ctx->offset += sizeof(struct psirpd_hdrs_bitmap); /* temporary */

    left = pkt_ctx->pkt_len - (pkt_ctx->offset - pkt_ctx->pkt);
#if 0
    maxsize = psirpd_bmap_maxsize_in_bytes(data_map);
#else
    maxsize = 0; /* XXX */
#endif

    PSIRP_ETL(left < 0, PSIRP_FAIL_OUT_OF_BUFFER, PSIRP_DBG_ERR,
               "Offset '%d' higher than lenght '%d' of the buffer\n",
               pkt_ctx->offset - pkt_ctx->pkt, pkt_ctx->pkt_len);

    PSIRP_MALLOC(tmpmap, maxsize);

#if 0
    err = psirpd_bmap_export(data_map, tmpmap, maxsize);
#else
    err = PSIRP_OK; /* XXX */
#endif
    if (err != PSIRP_OK) {
        PSIRP_DEBUG(PSIRP_DBG_ERR, "Unable to export bitmap");
        pkt_ctx->offset -= sizeof(struct psirpd_hdrs_bitmap);
	PSIRP_FREE(tmpmap);
        RETURN PSIRP_FAIL_BMAP;
    }

#if 0
    size = psirpd_bmap_rle_encode(tmpmap, maxsize, left);
#else
    size = 0; /* XXX */
#endif
    if (size > 0) {
        if (size < left) {
            /* Great success! */
	    memcpy(pkt_ctx->bmap_hdr->bitmap, tmpmap, size);
            pkt_ctx->bmap_hdr->compression = PSIRP_COMP_RLE;
            pkt_ctx->bmap_hdr->bitmap_len = htons(size);
        } else {
            /* RLE encoded, but too long *doh* */
            PSIRP_DEBUG(PSIRP_DBG_ERR, "How to recover?");
            abort();
        }
    } else {
        /* RLE encoding failed */
        pkt_ctx->bmap_hdr->compression = PSIRP_COMP_NOTUSED;
        pkt_ctx->bmap_hdr->bitmap_len = htons(maxsize);
    }

    PSIRP_FREE(tmpmap);
    pkt_ctx->offset += (PSIRPD_BITMAP_HDRLEN(pkt_ctx->bmap_hdr) - 
                        sizeof(struct psirpd_hdrs_bitmap));

    psirpd_hdrs_init_hdr(pkt_ctx, (psirpd_hdrs_t*)pkt_ctx->bmap_hdr, hdr_type);

#if 0
    PSIRP_DEBUG(PSIRP_DBG_GARB,"Bitmap length: %d bytes", 
		htons(pkt_ctx->bmap_hdr->bitmap_len));
#endif

    RETURN PSIRP_OK;
}
