/*
* Copyright (C) 2009, Oy L M Ericsson Ab, NomadicLab <pubsub@nomadiclab.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* Alternatively, this software may be distributed under the terms of the BSD
* license.
*
* See LICENSE and COPYING for more details.
*/ 

%inline %{
PyObject * psirp_py_data_to_buffer(caddr_t data, u_int64_t len) {
    return PyBuffer_FromReadWriteMemory(data, len);
}

PyObject * psirp_py_buffer(psirp_pub_t pub) {
    return psirp_py_data_to_buffer(psirp_pub_data(pub),
                                   psirp_pub_data_len(pub));
}

enum psirp_pub_type psirp_py_type(psirp_pub_t pub) {
    return psirp_pub_type(pub);
}

int psirp_py_subscribe_sync_(psirp_id_t *sid, psirp_id_t *rid,
			     psirp_pub_t *pubp, double timeout) {
    
    struct timeval tv_timeout, *tv_timeout_p;
    long sec, usec;
    
    if (timeout < 0) {
	tv_timeout_p = NULL;
    }
    else {
	sec = (long)timeout;
	usec = (long)((timeout-sec)*1000000);
	
	memset(&tv_timeout, 0, sizeof(tv_timeout));
	tv_timeout.tv_sec = (time_t)sec;
	tv_timeout.tv_usec = (suseconds_t)usec;	
	
	tv_timeout_p = &tv_timeout;
    }
    
    return psirp_subscribe_sync(sid, rid, pubp, tv_timeout_p);
}
%}

%constant int PSIRP_ID_LEN   = PSIRP_ID_LEN;
%constant int NOTE_PUBLISH   = NOTE_PUBLISH;
%constant int NOTE_SUBSCRIBE = NOTE_SUBSCRIBE;
%constant int NOTE_UNMAP     = NOTE_UNMAP;

%include "../include/libpsirp.h"


%inline %{
void psirp_py_debug_print_(int level, const char* file, const char *func,
			   const char *msg) {
    psirp_debug_print(level, file, func, (char *)msg);
}

void psirp_py_debug_enter_(const char* file, const char* func, int lineno) {
    psirp_debug_print(PSIRP_DBG_FNCE, file, func,
		      "%s(%d): ENTER\n", file, lineno);
}
%}

%include "../src/psirp_debug.h"
