#
# Copyright (C) 2009, Oy L M Ericsson Ab, NomadicLab <pubsub@nomadiclab.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# Alternatively, this software may be distributed under the terms of the BSD
# license.
#
# See LICENSE and COPYING for more details.
#


from psirp.libpsirp_py import *
import errno, sys


class Publication(object):
    
    def __init__(self, pub):
        self.__pub = pub
    
    def free(self):
        psirp_free(self.__pub)
    
    @property
    def pub(self):
        return self.__pub
    
    @property
    def buffer(self):
        return psirp_py_buffer(self.__pub)

    @property
    def data(self):
        return psirp_pub_data(self.__pub)
    
    @property
    def len(self):
        return psirp_pub_data_len(self.__pub)
    
    @property
    def rid(self):
        return psirp_pub_rid(self.__pub)
    
    @property
    def fd(self):
        return psirp_pub_fd(self.__pub)

    @property
    def type(self):
        return psirp_py_type(self.__pub)

    @property
    def addr(self):
        return int(self.__pub) % 2**31 # XXX


def _psirp_py_f(f, *args):
    v = f(*args)
    
    if v is None or len(v) < 2 or v[0] != 0 or v[1] is None:
        raise EnvironmentError(
            (v[0] if v is not None and len(v) >= 1 else 0),
            "psirp_[create|subscribe|subscribe_sync]() failed")
    
    py_pub = Publication(v[1])
    py_pub.__del__ = py_pub.free # XXX
    
    return py_pub

def psirp_py_create(len):
    return _psirp_py_f(psirp_create, len)

def psirp_py_subscribe(sid, rid):
    return _psirp_py_f(psirp_subscribe, sid, rid)

def psirp_py_subscribe_sync(sid, rid, timeout=None):
    return _psirp_py_f(psirp_py_subscribe_sync_, sid, rid,
                       (timeout if timeout is not None else -1))

def psirp_py_publish(sid, rid, pub):
    v = psirp_publish(sid, rid, pub.pub);
    
    if v != 0:
        raise EnvironmentError(v, "psirp_publish() failed")

def psirp_py_notfound(exception): # XXX
    return type(exception) is EnvironmentError \
        and exception.errno in (errno.ENOENT, errno.ESRCH)


def psirp_py_atoid(str):
    """Conversion function that corresponds to psirp_atoid()."""
    import binascii
    
    if not (len(str) % 2 == 0 \
                and ("::" in str or len(str) == PSIRP_ID_LEN*2)):
        raise ValueError("Bad Id string: bad length")
    
    substrings = str.split("::")
    if len(substrings) > 2:
        raise ValueError("Bad Id string: \"::\" occurs more than once")
    for ss in substrings:
        if len(ss) % 2 != 0:
            raise ValueError("Bad Id string: contains single digits")
    
    if len(substrings) == 2:
        str_ = ((PSIRP_ID_LEN*2-len(str)+2)*'0').join(substrings)
    else:
        str_ = substrings[0]
    
    binid = binascii.unhexlify(str_)
    assert len(binid) == PSIRP_ID_LEN
    
    return binid


def psirp_py_debug_init(level=0, colors=False):
    psirp_debug_init_print(level, 0)
    if colors:
        psirp_debug_printcols()

def psirp_py_debug_print(level, msg):
    frame = sys._getframe(1) # caller's frame
    psirp_py_debug_print_(level,
                          frame.f_code.co_filename,
                          frame.f_code.co_name,
                          msg)

def psirp_py_debug_enter():
    frame = sys._getframe(1) # caller's frame    
    psirp_py_debug_enter_(frame.f_code.co_filename,
                          frame.f_code.co_name,
                          frame.f_lineno)
